import { Button } from '@material-ui/core'
import LeftArrowIcon from 'assets/icons/ico/left-arrow.svg'
import MaxPowerIcon from 'assets/icons/ico/maxPower.svg'
import MinIcon from 'assets/icons/ico/minimum.svg'
import OffPeakHourIcon from 'assets/icons/ico/offPeakHour.svg'
import RightArrowIcon from 'assets/icons/ico/right-arrow.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { FluidType, TimeStep } from 'enums'
import {
  AggregatedEnedisMonthlyDataloads,
  EnedisMonthlyAnalysisData,
  FluidPrice,
  PerformanceIndicator,
} from 'models'
import React, { useCallback, useEffect, useMemo, useState } from 'react'
import ConsumptionService from 'services/consumption.service'
import EnedisMonthlyAnalysisDataService from 'services/enedisMonthlyAnalysisData.service'
import FluidPricesService from 'services/fluidsPrices.service'
import { useAppSelector } from 'store/hooks'
import { getNavPicto } from 'utils/picto'
import { OffPeakHours } from 'utils/utils'
import ElecHalfHourChart from './ElecHalfHourChart'
import ElecInfoModal from './ElecInfoModal'
import './elecHalfHourMonthlyAnalysis.scss'

const ElecHalfHourMonthlyAnalysis = ({
  perfIndicator,
}: {
  perfIndicator: PerformanceIndicator
}) => {
  const { t } = useI18n()
  const client = useClient()
  const { analysisMonth } = useAppSelector(state => state.ecolyo.analysis)

  const [isWeekend, setIsWeekend] = useState<boolean>(true)
  const [isHalfHourActivated, setIsHalfHourActivated] = useState<boolean>(false)
  const [isLoading, setIsLoading] = useState<boolean>(true)
  const [monthDataloads, setMonthDataloads] =
    useState<AggregatedEnedisMonthlyDataloads>()
  const [enedisAnalysisValues, setEnedisAnalysisValues] =
    useState<EnedisMonthlyAnalysisData>()
  const [facturePercentage, setFacturePercentage] = useState<number>()
  const [elecPrice, setElecPrice] = useState<FluidPrice>()
  const [openInfoModal, setOpenInfoModal] = useState<boolean>(false)
  const [offPeakHours, setOffPeakHours] = useState<OffPeakHours[]>()

  /** EnedisMonthlyAnalysisDataService */
  const emas = useMemo(
    () => new EnedisMonthlyAnalysisDataService(client),
    [client]
  )

  const isDataFullyComplete = useCallback(
    (monthDataloads: AggregatedEnedisMonthlyDataloads | undefined) => {
      return (
        monthDataloads?.weekend &&
        monthDataloads.week &&
        monthDataloads.weekend[0]?.value !== null &&
        monthDataloads.week[0]?.value !== null
      )
    },
    []
  )

  const getPowerChart = useCallback((): JSX.Element => {
    if (monthDataloads && isDataFullyComplete(monthDataloads)) {
      return (
        <ElecHalfHourChart
          dataLoad={isWeekend ? monthDataloads.weekend : monthDataloads.week}
          isWeekend={isWeekend}
        />
      )
    }
    return <p className="text-20-bold no_data">{t('analysis.no_data')}</p>
  }, [isDataFullyComplete, isWeekend, monthDataloads, t])

  useEffect(() => {
    let subscribed = true
    async function fetchEnedisAnalysisData() {
      const cs = new ConsumptionService(client)
      const isHalfHourLoadActivated = await cs.checkDoctypeEntries(
        FluidType.ELECTRICITY,
        TimeStep.HALF_AN_HOUR
      )
      if (!subscribed) return
      if (isHalfHourLoadActivated) {
        setIsHalfHourActivated(true)
        const aggregatedDate = analysisMonth.minus({ month: 1 })
        const data = await emas.getEnedisMonthlyAnalysisByDate(
          aggregatedDate.year,
          aggregatedDate.month
        )
        if (data?.length) {
          const aggregatedData = emas.aggregateValuesToDataLoad(data[0])
          setEnedisAnalysisValues(data[0])
          setMonthDataloads(aggregatedData)
          if (data[0].minimumLoad && perfIndicator.value) {
            const percentage = (data[0].minimumLoad / perfIndicator.value) * 100
            setFacturePercentage(percentage)
          }
        }
      }

      setIsLoading(false)
    }
    fetchEnedisAnalysisData()

    return () => {
      subscribed = false
    }
  }, [analysisMonth, client, emas, perfIndicator])

  useEffect(() => {
    let subscribed = true
    const fluidsPricesService = new FluidPricesService(client)
    async function getAllLastPrices() {
      const price = await fluidsPricesService.getPrices(
        FluidType.ELECTRICITY,
        analysisMonth.minus({ month: 1 })
      )
      if (subscribed && price) {
        setElecPrice(price)
      }
    }
    getAllLastPrices()

    return () => {
      subscribed = false
    }
  }, [analysisMonth, client])

  useEffect(() => {
    let subscribed = true
    async function getOffPeakHours() {
      if (subscribed) {
        const offPeakHours = await emas.getOffPeakHours()
        if (offPeakHours) {
          setOffPeakHours(offPeakHours)
        }
      }
    }
    getOffPeakHours()

    return () => {
      subscribed = false
    }
  }, [emas])

  return (
    <div className="special-elec-container">
      <StyledIcon
        className="elec-icon"
        icon={getNavPicto(FluidType.ELECTRICITY, true, true)}
        size={42}
      />
      <div className="text-18-normal title">{t('special_elec.title')}</div>
      {!isHalfHourActivated && (
        <div className="activation-text text-18-normal">
          {t(`timestep.half_an_hour.analysis_waiting_data`)}
        </div>
      )}
      {isHalfHourActivated && (
        <>
          <div className="navigator">
            <StyledIconButton
              icon={LeftArrowIcon}
              sized={24}
              onClick={() => setIsWeekend(prev => !prev)}
              aria-label={t('consumption.accessibility.button_previous_value')}
              className="arrow-prev"
            />
            <div className="average text-18-normal">
              <div className="text-1">{t('special_elec.average')}</div>
              <div className="text-2 text-18-bold">
                {t('special_elec.weektype')}{' '}
                <span className={isWeekend ? 'weekend' : 'week'}>
                  {isWeekend
                    ? t('special_elec.weekend')
                    : t('special_elec.week')}
                </span>
              </div>
            </div>
            <StyledIconButton
              icon={RightArrowIcon}
              sized={24}
              onClick={() => setIsWeekend(prev => !prev)}
              aria-label={t('consumption.accessibility.button_next_value')}
              className="arrow-next"
            />
          </div>
          {isLoading && <Loader color="elec" />}
          {!isLoading && (
            <>
              {getPowerChart()}
              {enedisAnalysisValues && isDataFullyComplete(monthDataloads) && (
                <>
                  <div className="elec-infos">
                    <div className="container">
                      <StyledIcon
                        icon={MaxPowerIcon}
                        size={40}
                        className="minIcon"
                      />
                      <div className="text text-18-normal">
                        <div className="row">
                          {t('special_elec.maxPower')}
                          {enedisAnalysisValues.maxPower !== 0 &&
                          enedisAnalysisValues.maxPower !== null ? (
                            <div className="value">
                              <div className="text-18-bold">
                                {enedisAnalysisValues.maxPower.toFixed(1)}
                              </div>
                              <span> kVA</span>
                            </div>
                          ) : (
                            <span>----</span>
                          )}
                        </div>
                      </div>
                    </div>
                    {enedisAnalysisValues?.offPeakHoursRatio != null && (
                      <div className="container">
                        <StyledIcon
                          icon={OffPeakHourIcon}
                          size={40}
                          className="minIcon"
                        />
                        <div className="text text-18-normal">
                          <div className="row">
                            {t('special_elec.offPeakHour')}
                            <div className="value">
                              <span className="text-18-bold">
                                {Math.round(
                                  enedisAnalysisValues.offPeakHoursRatio * 100
                                )}
                              </span>
                              <span> %</span>
                            </div>
                          </div>
                        </div>
                      </div>
                    )}
                    <div className="container">
                      <StyledIcon
                        icon={MinIcon}
                        size={40}
                        className="minIcon"
                      />
                      <div className="text text-18-normal">
                        {enedisAnalysisValues.minimumLoad !== 0 &&
                        enedisAnalysisValues.minimumLoad !== null ? (
                          <>
                            <div className="row">
                              <div>{t('special_elec.min')}</div>
                              <div className="value">
                                <span className="text-18-bold">
                                  {enedisAnalysisValues.minimumLoad.toFixed(1)}
                                </span>
                                <span> kWh</span>
                              </div>
                            </div>
                            <div className="row">
                              <div>{t('special_elec.percentage')}</div>
                              <div className="value">
                                <span className="text-18-bold">
                                  {facturePercentage?.toFixed(0) || '---'}
                                </span>
                                <span> %</span>
                              </div>
                            </div>
                            <div className="row">
                              <div>{t('special_elec.price')}</div>
                              <div className="value">
                                <span className="text-18-bold">
                                  {elecPrice?.price
                                    ? (
                                        enedisAnalysisValues.minimumLoad *
                                        elecPrice.price
                                      ).toFixed(1)
                                    : '---'}
                                </span>
                                <span> €</span>
                              </div>
                            </div>
                          </>
                        ) : (
                          <span>----</span>
                        )}
                      </div>
                    </div>
                  </div>

                  <Button
                    className="btnText"
                    onClick={() => setOpenInfoModal(true)}
                  >
                    {t('special_elec.showModal')}
                  </Button>
                </>
              )}
            </>
          )}
        </>
      )}
      <ElecInfoModal
        open={openInfoModal}
        offPeakHours={offPeakHours}
        handleCloseClick={() => setOpenInfoModal(false)}
      />
    </div>
  )
}

export default ElecHalfHourMonthlyAnalysis
