import { Button } from '@material-ui/core'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import EstimatedConsumptionModal from 'components/ConsumptionVisualizer/EstimatedConsumptionModal'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { FluidType, TimeStep } from 'enums'
import { DataloadValueDetail, TimePeriod } from 'models'
import React, { useEffect, useState } from 'react'
import ConsumptionDataManager from 'services/consumption.service'
import { useAppSelector } from 'store/hooks'
import { getNavPicto } from 'utils/picto'
import { formatNumberValues, getMonthNameWithPrep } from 'utils/utils'
import PieChart from './PieChart'
import './totalAnalysisChart.scss'

const TotalAnalysisChart = ({
  fluidsWithData,
}: {
  fluidsWithData: FluidType[]
}) => {
  const { t } = useI18n()
  const client = useClient()
  const { analysisMonth } = useAppSelector(state => state.ecolyo.analysis)
  const [isLoading, setIsLoading] = useState(true)
  const [dataLoadValueDetailArray, setDataLoadValueDetailArray] = useState<
    DataloadValueDetail[] | null
  >(null)
  const [totalLoadValue, setTotalLoadValue] = useState<number>(0)
  const [openEstimationModal, setOpenEstimationModal] = useState<boolean>(false)
  const arcWidth = 30
  const radius = Math.min(375, innerWidth - 100)
  const outerRadius = radius / 2
  const innerRadius = outerRadius - arcWidth

  useEffect(() => {
    let subscribed = true
    async function getTotalData() {
      const timePeriod: TimePeriod = {
        startDate: analysisMonth.minus({ month: 1 }).startOf('month'),
        endDate: analysisMonth.minus({ month: 1 }).endOf('month'),
      }
      const consumptionService = new ConsumptionDataManager(client)
      const monthTotalData = await consumptionService.getGraphData({
        timePeriod,
        timeStep: TimeStep.MONTH,
        fluidTypes: fluidsWithData,
        isHome: true,
      })
      if (monthTotalData?.actualData) {
        setDataLoadValueDetailArray(monthTotalData.actualData[0].valueDetail)
        setTotalLoadValue(monthTotalData.actualData[0].value)
      }
      setIsLoading(false)
    }
    if (subscribed) {
      getTotalData()
    }
    return () => {
      subscribed = false
    }
  }, [analysisMonth, client, fluidsWithData])

  const emptyPieChart = () => (
    <PieChart
      dataloadValueDetailArray={[]}
      radius={radius}
      innerRadius={innerRadius}
      outerRadius={outerRadius}
    >
      <div className="text-36-bold">
        {formatNumberValues(0)}
        <span className="euro-unit">{t('FLUID.MULTIFLUID.UNIT')}</span>
      </div>
      <div className="text-20-bold no_data">{t('analysis.no_data')}</div>
    </PieChart>
  )

  return (
    <>
      <div
        className="totalAnalysis-container"
        style={{
          minHeight: radius + 100,
        }}
      >
        <div className="text-24-normal title">{t('analysis_pie.total')}</div>

        {isLoading && <Loader />}
        {!isLoading && !dataLoadValueDetailArray && emptyPieChart()}
        {!isLoading && dataLoadValueDetailArray && (
          <>
            <PieChart
              dataloadValueDetailArray={dataLoadValueDetailArray}
              radius={radius}
              innerRadius={innerRadius}
              outerRadius={outerRadius}
            >
              <div className="text-36-bold">
                {formatNumberValues(totalLoadValue)}
                <span className="euro-unit">{t('FLUID.MULTIFLUID.UNIT')}</span>
              </div>
              <div className="text-16-normal date">
                {t('analysis_pie.month') +
                  getMonthNameWithPrep(analysisMonth.minus({ month: 1 }))}
              </div>
              <Button
                className="btnText"
                onClick={() => setOpenEstimationModal(true)}
              >
                <span
                  className="estimated"
                  dangerouslySetInnerHTML={{
                    __html: t('analysis_pie.estimation'),
                  }}
                />
              </Button>
            </PieChart>
            <EstimatedConsumptionModal
              open={openEstimationModal}
              handleCloseClick={() => setOpenEstimationModal(false)}
            />
            {dataLoadValueDetailArray && fluidsWithData.length > 1 && (
              <div
                role="list"
                aria-label={t('analysis_pie.details')}
                className="total-card-container"
              >
                {dataLoadValueDetailArray.map((dataload, index) => (
                  <div key={index} role="listitem" className="total-card">
                    <div className="text-18-bold fluidconso">
                      {dataload.value !== -1
                        ? `${formatNumberValues(dataload.value)} €`
                        : '--- €'}
                    </div>
                    <StyledIcon
                      className="euro-fluid-icon"
                      icon={getNavPicto(index, true, true)}
                      size={38}
                    />
                    <div className="text-16-normal">
                      {t(`FLUID.${FluidType[index]}.LABEL`)}
                    </div>
                  </div>
                ))}
              </div>
            )}
          </>
        )}
      </div>
    </>
  )
}

export default TotalAnalysisChart
