import CaptionAverageIcon from 'assets/icons/visu/duel/captionAverage.svg'
import CaptionConsumptionIcon from 'assets/icons/visu/duel/captionConsumption.svg'
import CaptionIncomingIcon from 'assets/icons/visu/duel/captionIncoming.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import { useChartResize } from 'components/Hooks/useChartResize'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import {
  UserChallengeSuccess,
  UserChallengeUpdateFlag,
  UserDuelState,
} from 'enums'
import { Dataload, UserChallenge } from 'models'
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react'
import { useNavigate } from 'react-router-dom'
import ChallengeService from 'services/challenge.service'
import {
  unlockNextUserChallenge,
  updateUserChallengeList,
} from 'store/challenge/challenge.slice'
import { toggleChallengeDuelNotification } from 'store/global/global.slice'
import { useAppDispatch, useAppSelector } from 'store/hooks'
import { formatNumberValues } from 'utils/utils'
import DuelChart from '../DuelChart/DuelChart'
import DuelResultModal from '../DuelResultModal/DuelResultModal'
import LastDuelModal from '../LastDuelModal/lastDuelModal'
import './duelOngoing.scss'

interface DuelOngoingProps {
  userChallenge: UserChallenge
  isFinished?: boolean
}

const DuelOngoing = ({ userChallenge, isFinished }: DuelOngoingProps) => {
  const { t } = useI18n()
  const client = useClient()
  const navigate = useNavigate()
  const dispatch = useAppDispatch()
  const { currentDataload, userChallengeList } = useAppSelector(
    state => state.ecolyo.challenge
  )
  const [resultModal, setResultModal] = useState<boolean>(false)
  const [winChallenge, setWinChallenge] = useState<boolean>(false)
  const [showLastDuelModal, setShowLastDuelModal] = useState<boolean>(false)
  const [finishedDataLoad, setFinishedDataLoad] = useState<Dataload[]>()
  const chartContainer = useRef<HTMLDivElement>(null)
  const { height, width } = useChartResize(chartContainer, false)
  const challengeService = useMemo(() => new ChallengeService(client), [client])

  const duel = userChallenge.duel

  const isLastDuel =
    userChallenge.id == userChallengeList[userChallengeList.length - 1]?.id

  const userConsumption = formatNumberValues(
    userChallenge.duel.userConsumption
  ).toString()
  const average = formatNumberValues(userChallenge.duel.threshold).toString()

  const setResult = useCallback(async () => {
    const updatedChallenge = await challengeService.updateUserChallenge(
      userChallenge,
      winChallenge
        ? UserChallengeUpdateFlag.DUEL_WIN
        : UserChallengeUpdateFlag.DUEL_LOSS
    )
    dispatch(updateUserChallengeList(updatedChallenge))
    dispatch(unlockNextUserChallenge(updatedChallenge))
    dispatch(toggleChallengeDuelNotification(false))

    if (isLastDuel) {
      setShowLastDuelModal(true)
    } else {
      navigate('/challenges')
    }
  }, [
    challengeService,
    userChallenge,
    winChallenge,
    dispatch,
    isLastDuel,
    navigate,
  ])

  useEffect(() => {
    let subscribed = true
    async function setChallengeResult() {
      const { isDone, isWin } = await challengeService.isChallengeDone(
        userChallenge,
        currentDataload
      )
      if (subscribed) {
        setResultModal(isDone)
        setWinChallenge(isWin)
      }
    }
    setChallengeResult()
    return () => {
      subscribed = false
    }
  }, [challengeService, client, currentDataload, userChallenge])

  useEffect(() => {
    let subscribed = true
    async function populateData() {
      if (isFinished) {
        const dataloads =
          await challengeService.getUserChallengeDataload(userChallenge)
        if (subscribed) {
          setFinishedDataLoad(dataloads)
        }
      }
    }
    populateData()
    return () => {
      subscribed = false
    }
  }, [challengeService, dispatch, isFinished, userChallenge])

  return (
    <>
      <div className="duel-ongoing-container">
        <div className="duel-title text-16-normal">{duel.title}</div>
        {duel.state === UserDuelState.DONE ? (
          <div className="duel-goal text-26-bold">
            {userChallenge.success === UserChallengeSuccess.WIN
              ? t('duel.goal_done')
              : t('duel.goal_failed')}
          </div>
        ) : (
          <div className="duel-goal text-18-bold">
            {t('duel.goal1', {
              durationInDays: duel.duration.days,
            })}
            <span> </span>
            {t('duel.goal2', {
              title: duel.title,
            })}
          </div>
        )}
        <div className="duel-consumption text-28-normal">
          <span className="consumption">{userConsumption}</span>
          {` / ${average} €`}
        </div>
        <div className="duel-chart fs-root" ref={chartContainer}>
          <DuelChart
            userChallenge={userChallenge}
            finishedDataLoad={finishedDataLoad}
            width={width}
            height={height}
          />
        </div>
        <div className="duel-chart-caption">
          <div className="duel-caption">
            <StyledIcon
              className="caption-icon"
              icon={CaptionAverageIcon}
              size={25}
            />
            <span>{t('duel.caption_average')}</span>
          </div>
          <div className="duel-caption">
            <StyledIcon
              className="caption-icon"
              icon={CaptionConsumptionIcon}
              size={25}
            />
            <span>{t('duel.caption_consumption')}</span>
          </div>
          <div className="duel-caption">
            <StyledIcon
              className="caption-icon"
              icon={CaptionIncomingIcon}
              size={25}
            />
            <span>{t('duel.caption_incoming')}</span>
          </div>
        </div>
      </div>
      <DuelResultModal
        open={resultModal}
        userChallenge={userChallenge}
        win={winChallenge}
        handleCloseClick={setResult}
      />
      <LastDuelModal
        open={showLastDuelModal}
        handleCloseClick={() => navigate('/challenges')}
      />
    </>
  )
}

export default DuelOngoing
