import { Grow, ListItemIcon, Menu, MenuItem } from '@material-ui/core/'
import Button from '@material-ui/core/Button'
import CheckIcon from 'assets/icons/ico/check.svg'
import SortIcon from 'assets/icons/ico/sort.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import EcogestureCard from 'components/Ecogesture/EcogestureCard/EcogestureCard'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { Usage } from 'enums'
import { Ecogesture } from 'models'
import React, { useState } from 'react'
import { useNavigate } from 'react-router-dom'
import { updateEcogestureFilter } from 'store/global/global.slice'
import { useAppDispatch, useAppSelector } from 'store/hooks'
import './ecogestureList.scss'

interface EcogestureListProps {
  list: Ecogesture[]
  displaySelection: boolean
  selectionTotal: number
  selectionViewed: number
  openResetEcogestureModal?: () => void
}

const EcogestureList = ({
  list,
  displaySelection,
  selectionTotal,
  selectionViewed,
  openResetEcogestureModal,
}: EcogestureListProps) => {
  const { t } = useI18n()
  const navigate = useNavigate()
  const dispatch = useAppDispatch()
  const { ecogestureFilter } = useAppSelector(state => state.ecolyo.global)
  const [openDropDown, setOpenDropDown] = useState<boolean>(false)
  const [anchorEl, setAnchorEl] = useState<null | HTMLElement>(null)

  const toggleMenu = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget)
    setOpenDropDown(true)
  }

  const closeMenu = (usage?: Usage) => {
    usage !== undefined && dispatch(updateEcogestureFilter(usage))
    setAnchorEl(null)
    setOpenDropDown(false)
  }

  const renderEcogestureContent = () => {
    const isAllUsage = ecogestureFilter === Usage.ALL
    const filteredEcogestures = list.filter(ecogesture =>
      isAllUsage ? true : ecogesture.usage === ecogestureFilter
    )

    if (filteredEcogestures.length > 0) {
      return filteredEcogestures.map(ecogesture => (
        <EcogestureCard key={ecogesture.id} ecogesture={ecogesture} />
      ))
    }

    return (
      <div className="ec-filter-error">
        <div className="text-20-normal">
          {t('ecogesture.no_ecogesture_filter.text1')}
        </div>
        <div className="text-16-italic">
          {t('ecogesture.no_ecogesture_filter.text2')}
        </div>
      </div>
    )
  }

  const selectFilters = () => (
    <>
      <Button
        className="btnSecondary btnFilter"
        aria-controls="simple-menu"
        aria-haspopup="true"
        aria-label={t(`ecogesture.MENU_TITLE`)}
        onClick={toggleMenu}
        size="medium"
      >
        <StyledIcon icon={SortIcon} size={20} />
        <span className={openDropDown ? 'opened' : ''}>
          {ecogestureFilter === Usage.ALL
            ? t(`ecogesture.MENU_TITLE`)
            : t(`ecogesture.${Usage[ecogestureFilter]}`)}
        </span>
      </Button>
      <Menu
        anchorEl={anchorEl}
        keepMounted
        open={Boolean(anchorEl)}
        onClose={() => closeMenu()}
        PopoverClasses={{
          paper: 'filter-menu',
        }}
        variant="menu"
        MenuListProps={{ className: 'filter-menu-list' }}
      >
        {Object.values(Usage).map(usage => {
          const active = usage === ecogestureFilter
          return (
            typeof usage === 'number' && (
              <MenuItem
                key={usage}
                classes={{
                  root: `${active ? 'item-active' : ''}`,
                }}
                selected={active}
                onClick={() => closeMenu(usage)}
              >
                {t(`ecogesture.${Usage[usage]}`)}
                {active && (
                  <ListItemIcon classes={{ root: 'filter-menu-icon' }}>
                    <StyledIcon icon={CheckIcon} size={13} />
                  </ListItemIcon>
                )}
              </MenuItem>
            )
          )
        })}
      </Menu>
    </>
  )

  const continueSelection = () => (
    <Grow in={displaySelection}>
      <div className="selection text-16-normal">
        <div>
          <div>{t('ecogesture.selection')}</div>
          <div>
            {`(${selectionViewed} ${t(
              'ecogesture.selection_2'
            )} ${selectionTotal})`}
          </div>
        </div>
        <Button
          aria-label={t('ecogesture.accessibility.button_selection')}
          className="btnPrimary"
          onClick={() => navigate('/ecogesture-selection')}
        >
          {t('ecogesture.button_selection')}
        </Button>
      </div>
    </Grow>
  )

  return (
    <div className="ecogesture-root">
      <div className="efficiency-button-content">
        {selectFilters()}
        {displaySelection && continueSelection()}
      </div>

      <div role="list" className="ecogesture-content">
        {renderEcogestureContent()}
        {!displaySelection && openResetEcogestureModal && (
          <Button onClick={openResetEcogestureModal} className="btnSecondary">
            {t('ecogesture.reset')}
          </Button>
        )}
      </div>
    </div>
  )
}

export default EcogestureList
