import {
  Accordion,
  AccordionDetails,
  AccordionSummary,
  Button,
} from '@material-ui/core'
import chevronDown from 'assets/icons/ico/chevron-down.svg'
import ErrorNotif from 'assets/icons/ico/notif_error.svg'
import PartnersIssueNotif from 'assets/icons/ico/notif_maintenance.svg'
import WarningNotif from 'assets/icons/ico/notif_warning.svg'
import OfflinePicto from 'assets/icons/visu/offline-param.svg'
import classNames from 'classnames'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import Connection from 'components/Connection/Connection'
import { GrdfWaitConsent } from 'components/Connection/GRDFConnect/GrdfWaitConsent'
import KonnectorModal from 'components/Konnector/KonnectorModal'
import { useClient } from 'cozy-client'
import { isKonnectorRunning } from 'cozy-harvest-lib/dist/helpers/triggers'
import ConnectionFlow from 'cozy-harvest-lib/dist/models/ConnectionFlow'
import {
  ERROR_EVENT,
  LOGIN_SUCCESS_EVENT,
  SUCCESS_EVENT,
} from 'cozy-harvest-lib/dist/models/flowEvents'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import {
  FluidSlugType,
  FluidState,
  FluidType,
  KonnectorError,
  UserChallengeState,
  UserDuelState,
} from 'enums'
import { DateTime } from 'luxon'
import { AccountEGLData, FluidConnection, FluidStatus } from 'models'
import React, { useCallback, useEffect, useMemo, useState } from 'react'
import { useNavigate } from 'react-router-dom'
import AccountService from 'services/account.service'
import ChallengeService from 'services/challenge.service'
import DateChartService from 'services/dateChart.service'
import FluidService from 'services/fluid.service'
import PartnersInfoService from 'services/partnersInfo.service'
import { setChallengeConsumption } from 'store/challenge/challenge.slice'
import {
  setSelectedDate,
  setShowConnectionDetails,
  setShowOfflineData,
} from 'store/chart/chart.slice'
import {
  setFluidStatus,
  setLastEpglLogin,
  toggleChallengeDuelNotification,
  updateFluidConnection,
} from 'store/global/global.slice'
import { useAppDispatch, useAppSelector } from 'store/hooks'
import logApp from 'utils/logger'
import { getParamPicto } from 'utils/picto'
import { getKonnectorSlug } from 'utils/utils'
import ConnectionNotFound from './ConnectionNotFound/ConnectionNotFound'
import ConnectionResult from './ConnectionResult/ConnectionResult'
import './konnectorViewerCard.scss'

const KonnectorViewerCard = ({
  fluidType,
}: {
  fluidType: Exclude<FluidType, FluidType.MULTIFLUID>
}) => {
  const { t } = useI18n()
  const client = useClient()
  const navigate = useNavigate()
  const dispatch = useAppDispatch()
  const {
    chart: { showConnectionDetails, showOfflineData },
    challenge: { currentChallenge },
    global: { fluidStatus, shouldRefreshConsent, partnersInfo },
  } = useAppSelector(state => state.ecolyo)
  const currentFluidStatus = fluidStatus[fluidType]
  const fluidSlug = currentFluidStatus.connection.konnectorConfig.slug
  const fluidState = currentFluidStatus.status
  const { konnector, account, trigger } = currentFluidStatus.connection
  const currentFluidName = FluidType[currentFluidStatus.fluidType].toLowerCase()

  const [openModal, setOpenModal] = useState(false)
  const [isVerifyingIdentity, setIsVerifyingIdentity] = useState(
    fluidType === FluidType.ELECTRICITY
  )
  const [konnectorErrorDescription, setKonnectorErrorDescription] =
    useState<KonnectorError | null>(null)
  const [konnectorState, setKonnectorState] = useState<string | null>(null)
  const [isOutdatedData, setIsOutdatedData] = useState<number | null>(null)

  const isWaitingForConsent =
    fluidType === FluidType.GAS &&
    currentFluidStatus.status === FluidState.CHALLENGE_ASKED

  const fluidService = useMemo(() => new FluidService(client), [client])
  const accountService = useMemo(() => new AccountService(client), [client])
  const partnersInfoService = useMemo(
    () => new PartnersInfoService(client),
    [client]
  )

  const iconType = getParamPicto(currentFluidStatus.fluidType)

  const toggleAccordion = () => {
    dispatch(setShowConnectionDetails(!showConnectionDetails))
  }

  const updateGlobalFluidStatus = useCallback(async (): Promise<
    FluidStatus[]
  > => {
    const updatedFluidStatus = await fluidService.getFluidStatus()
    const refDate = DateTime.fromISO('0001-01-01')
    let lastDataDate = DateTime.fromISO('0001-01-01')
    for (const fluid of updatedFluidStatus) {
      if (fluid?.lastDataDate && fluid?.lastDataDate > lastDataDate) {
        lastDataDate = fluid.lastDataDate
      }
    }
    if (lastDataDate > refDate) {
      dispatch(setSelectedDate(lastDataDate))
    }

    return updatedFluidStatus
  }, [dispatch, fluidService])

  const refreshChallengeState = useCallback(async () => {
    if (
      currentChallenge?.state === UserChallengeState.DUEL &&
      currentChallenge?.duel.state === UserDuelState.ONGOING
    ) {
      const challengeService = new ChallengeService(client)
      const { updatedUserChallenge, dataloads } =
        await challengeService.initChallengeDuelProgress(currentChallenge)
      dispatch(
        setChallengeConsumption({
          userChallenge: updatedUserChallenge,
          currentDataload: dataloads,
        })
      )
      // Check is duel is done and display notification
      const { isDone } = await challengeService.isChallengeDone(
        updatedUserChallenge,
        dataloads
      )
      dispatch(toggleChallengeDuelNotification(isDone))
    }
  }, [client, dispatch, currentChallenge])

  const handleAccountDeletion = useCallback(async () => {
    await refreshChallengeState()
    const _updatedFluidStatus = await updateGlobalFluidStatus()
    if (_updatedFluidStatus.length > 0) {
      const partnersInfo = await partnersInfoService.getPartnersInfo()
      const updatedFluidStatus = await fluidService.getFluidStatus(partnersInfo)
      dispatch(setFluidStatus(updatedFluidStatus))
    }
    dispatch(setShowConnectionDetails(false))
  }, [
    refreshChallengeState,
    updateGlobalFluidStatus,
    partnersInfoService,
    fluidService,
    dispatch,
  ])

  /** Close modal, update fluid status and reset error state */
  const handleConnectionEnd = useCallback(
    async (isSuccess?: boolean) => {
      const { account, isUpdating } = currentFluidStatus.connection

      // CASE FOR GLOBAL LOGIN FAILED
      const isGlobalLoginFailed =
        konnectorErrorDescription === KonnectorError.LOGIN_FAILED ||
        konnectorErrorDescription === KonnectorError.UNKNOWN_ERROR ||
        konnectorErrorDescription === KonnectorError.CRITICAL ||
        konnectorErrorDescription === KonnectorError.MISSING_SECRET ||
        konnectorErrorDescription ===
          KonnectorError.USER_ACTION_NEEDED_ACCOUNT_REMOVED

      // CASE FOR ENEDIS CODE INSEE ERROR
      const isEnedisCodeInseeError =
        !isUpdating && fluidType === FluidType.ELECTRICITY

      const isGRDFError =
        fluidType === FluidType.GAS &&
        (konnectorErrorDescription === KonnectorError.MAINTENANCE ||
          konnectorErrorDescription === KonnectorError.VENDOR_DOWN ||
          konnectorErrorDescription ===
            KonnectorError.USER_ACTION_NEEDED_CGU_FORM)

      const shouldDeleteAccount =
        account &&
        !isSuccess &&
        (isGlobalLoginFailed || isEnedisCodeInseeError || isGRDFError)

      if (shouldDeleteAccount) {
        logApp('info', `shouldDeleteAccount`)
        // KEEP LAST LOGIN FOR EPGL
        if (fluidSlug === FluidSlugType.WATER && account?.auth) {
          const auth = account.auth as AccountEGLData
          const lastEpglLogin = auth.login
          dispatch(setLastEpglLogin(lastEpglLogin))
        }
        // DELETE ACCOUNT
        await accountService.deleteAccount(account)
        await handleAccountDeletion()
      } else {
        const updatedFluidStatus =
          await fluidService.getFluidStatus(partnersInfo)
        dispatch(setFluidStatus(updatedFluidStatus))
      }
      dispatch(setShowConnectionDetails(false))
      setOpenModal(false)
      setKonnectorState(null)
      setKonnectorErrorDescription(null)
    },
    [
      konnectorErrorDescription,
      fluidType,
      currentFluidStatus,
      dispatch,
      fluidSlug,
      accountService,
      handleAccountDeletion,
      fluidService,
      partnersInfo,
    ]
  )

  const toggleModalConnection = useCallback(() => {
    switch (fluidType) {
      case FluidType.ELECTRICITY:
        navigate('/connect/electricity')
        break
      case FluidType.GAS:
        navigate('/connect/gas')
        break
      case FluidType.WATER:
        dispatch(setShowOfflineData(false))
        break
    }
  }, [dispatch, fluidType, navigate])

  const getConnectionCard = useCallback(() => {
    if (showOfflineData && !account) {
      return (
        <Button className="btnPrimary" onClick={toggleModalConnection}>
          {t(`auth.${getKonnectorSlug(fluidType)}.connect`)}
        </Button>
      )
    }

    if (
      fluidState === FluidState.KONNECTOR_NOT_FOUND &&
      !currentFluidStatus.connection.isUpdating
    ) {
      return <ConnectionNotFound konnectorSlug={fluidSlug} />
    }

    // Handle login failed for EGL
    if (
      (fluidType === FluidType.WATER &&
        fluidState === FluidState.LOGIN_FAILED) ||
      (account && currentFluidStatus.status !== FluidState.NOT_CONNECTED)
    ) {
      return (
        <ConnectionResult
          handleAccountDeletion={handleAccountDeletion}
          fluidType={fluidType}
        />
      )
    }
  }, [
    account,
    currentFluidStatus.connection.isUpdating,
    currentFluidStatus.status,
    fluidSlug,
    fluidState,
    fluidType,
    handleAccountDeletion,
    showOfflineData,
    t,
    toggleModalConnection,
  ])

  const callbackResponse = useCallback(
    async (_state: string) => {
      if (_state !== LOGIN_SUCCESS_EVENT) {
        const updatedConnection: FluidConnection = {
          ...currentFluidStatus.connection,
          shouldLaunchKonnector: false,
        }
        dispatch(
          updateFluidConnection({
            fluidType: currentFluidStatus.fluidType,
            fluidConnection: updatedConnection,
          })
        )
        const [, updatedFluidStatus] = await Promise.all([
          await refreshChallengeState(),
          await updateGlobalFluidStatus(),
        ])

        /**
         * If waiting for consent, close Konnector modal first and handle connection closed
         * Calling handleConnectionEnd here because it won't be called in the KonnectorModal since we don't show it
         *
         * Else set error for KonnectorModal
         */
        if (
          fluidType === FluidType.GAS &&
          updatedFluidStatus[2].status == FluidState.CHALLENGE_ASKED
        ) {
          await handleConnectionEnd(false)
        } else {
          setKonnectorState(_state)
        }
      }
    },
    [
      currentFluidStatus.connection,
      currentFluidStatus.fluidType,
      dispatch,
      fluidType,
      handleConnectionEnd,
      refreshChallengeState,
      updateGlobalFluidStatus,
    ]
  )

  /** Display konnector icon & its smaller status icon in upper left corner */
  const displayKonnectorIcon = useCallback(() => {
    const { maintenance, status, connection } = currentFluidStatus
    let statusIcon = null

    if (maintenance) {
      statusIcon = PartnersIssueNotif
    } else if (
      (status === FluidState.ERROR || status === FluidState.LOGIN_FAILED) &&
      connection.account
    ) {
      statusIcon = ErrorNotif
    } else if (isOutdatedData && isOutdatedData > 0) {
      statusIcon = WarningNotif
    }

    return (
      <div className="konnector-icon">
        <StyledIcon
          icon={currentFluidStatus.connection.account ? iconType : OfflinePicto}
          size={49}
        />
        {statusIcon && (
          <StyledIcon
            icon={statusIcon}
            size={24}
            className="konnector-state-picto"
          />
        )}
      </div>
    )
  }, [currentFluidStatus, iconType, isOutdatedData])

  const displayKonnectorHeader = useCallback(() => {
    if (currentFluidStatus.maintenance) {
      return (
        <span className="text-16-bold">
          {t(`konnector_options.partner_issue`)}
        </span>
      )
    } else if (isOutdatedData && currentFluidStatus.connection.account) {
      return (
        <span className="text-16-bold outdated">
          {t('konnector_options.outdated', {
            isOutdatedData: isOutdatedData,
          })}
        </span>
      )
    } else if (currentFluidStatus.connection.account && !isOutdatedData) {
      return t(`FLUID.${currentFluidName.toUpperCase()}.LABEL`)
    } else {
      return t(`konnector_options.label_offline_${currentFluidName}`)
    }
  }, [
    currentFluidName,
    currentFluidStatus.connection.account,
    currentFluidStatus.maintenance,
    isOutdatedData,
    t,
  ])

  // If user has selected accept button on Expired consent modal, then delete its account
  useEffect(() => {
    async function deleteAccountForConsentRefresh() {
      if (shouldRefreshConsent && account) {
        await accountService.deleteAccount(account)
        await handleAccountDeletion()
      }
    }
    deleteAccountForConsentRefresh()
  }, [account, accountService, handleAccountDeletion, shouldRefreshConsent])

  useEffect(() => {
    let subscribed = true
    async function getData() {
      if (
        currentFluidStatus.connection.shouldLaunchKonnector &&
        !isKonnectorRunning(trigger)
      ) {
        if (subscribed) {
          setOpenModal(true)
          // Reset any error that might have been set after handleConnectionEnd
          setKonnectorErrorDescription(null)
          const updatedConnection: FluidConnection = {
            ...currentFluidStatus.connection,
            shouldLaunchKonnector: false,
          }
          dispatch(
            updateFluidConnection({
              fluidType: currentFluidStatus.fluidType,
              fluidConnection: updatedConnection,
            })
          )
        }
        const connectionFlow = new ConnectionFlow(client, trigger, konnector)
        await connectionFlow.launch()
        connectionFlow.jobWatcher.on(ERROR_EVENT, async () => {
          await callbackResponse(ERROR_EVENT)
          setKonnectorErrorDescription(connectionFlow.jobWatcher.on()._error)
        })
        connectionFlow.jobWatcher.on(LOGIN_SUCCESS_EVENT, () => {
          setIsVerifyingIdentity(false)
        })
        connectionFlow.jobWatcher.on(SUCCESS_EVENT, () => {
          callbackResponse(SUCCESS_EVENT)
        })
      }
    }
    !shouldRefreshConsent && getData()
    const dateChartService = new DateChartService()
    setIsOutdatedData(
      dateChartService.isDataOutdated(
        currentFluidStatus.lastDataDate,
        fluidType
      )
    )
    return () => {
      subscribed = false
    }
  }, [
    callbackResponse,
    client,
    currentFluidStatus.connection,
    currentFluidStatus.fluidType,
    currentFluidStatus.lastDataDate,
    dispatch,
    fluidSlug,
    fluidType,
    konnector,
    shouldRefreshConsent,
    trigger,
  ])

  return (
    <div className="konnector-section-root">
      {isWaitingForConsent && <GrdfWaitConsent />}
      {!isWaitingForConsent && !showOfflineData && (
        <Connection fluidType={fluidType} />
      )}
      {!isWaitingForConsent && showOfflineData && (
        <Accordion
          expanded={showConnectionDetails}
          onChange={toggleAccordion}
          classes={{
            root: classNames('expansion-panel-root', {
              ['red-border']:
                !currentFluidStatus.maintenance &&
                (currentFluidStatus.status === FluidState.ERROR ||
                  currentFluidStatus.status === FluidState.LOGIN_FAILED),
            }),
          }}
        >
          <AccordionSummary
            aria-label={t(
              `konnector_options.accessibility.button_toggle_detail_${currentFluidName}`
            )}
            expandIcon={
              <StyledIcon icon={chevronDown} className="accordion-icon" />
            }
            classes={{
              root: 'expansion-panel-summary',
              content: 'expansion-panel-content',
            }}
          >
            {displayKonnectorIcon()}
            <div
              className={classNames('text-16-bold konnector-title', {
                [`${currentFluidName}-connected`]:
                  currentFluidStatus.status !== FluidState.NOT_CONNECTED &&
                  !currentFluidStatus.maintenance,
              })}
            >
              {displayKonnectorHeader()}
            </div>
          </AccordionSummary>
          <AccordionDetails
            classes={{
              root: 'expansion-panel-details',
            }}
          >
            {getConnectionCard()}
          </AccordionDetails>
        </Accordion>
      )}
      <KonnectorModal
        open={openModal}
        isVerifyingIdentity={isVerifyingIdentity}
        state={konnectorState}
        error={konnectorErrorDescription}
        fluidType={fluidType}
        handleCloseClick={handleConnectionEnd}
        handleAccountDeletion={handleAccountDeletion}
        account={account}
      />
    </div>
  )
}
export default KonnectorViewerCard
