import Button from '@material-ui/core/Button'
import CloseIcon from 'assets/icons/ico/close.svg'
import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'
import QuizExplanationModal from 'components/Quiz/QuizExplanationModal/QuizExplanationModal'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { UserChallengeUpdateFlag } from 'enums'
import { UserChallenge } from 'models'
import React, { Dispatch, SetStateAction, useCallback, useState } from 'react'
import ChallengeService from 'services/challenge.service'
import QuizService from 'services/quiz.service'
import { updateUserChallengeList } from 'store/challenge/challenge.slice'
import { useAppDispatch } from 'store/hooks'
import './quizQuestion.scss'

interface QuizQuestionContent {
  userChallenge: UserChallenge
  setIsCustomQuest: Dispatch<SetStateAction<boolean>>
  goBack: () => void
}

const QuizQuestionContent = ({
  userChallenge,
  setIsCustomQuest,
  goBack,
}: QuizQuestionContent) => {
  const { t } = useI18n()
  const client = useClient()
  const dispatch = useAppDispatch()
  const questionIndexLocked = userChallenge.quiz.questions.findIndex(
    answer => answer.result == 0
  )
  const [userChoice, setUserChoice] = useState<string>('')
  const [openModal, setOpenModal] = useState<boolean>(false)
  const [answerIndex, setAnswerIndex] = useState<number>(0)
  const [questionIndex, setQuestionIndex] =
    useState<number>(questionIndexLocked)

  const quizService = new QuizService(client)
  const challengeService = new ChallengeService(client)

  const validateQuestion = async () => {
    const resultIndex = userChallenge.quiz.questions[
      questionIndex
    ].answers.findIndex(answer => answer.answerLabel === userChoice)
    const result = userChallenge.quiz.questions[questionIndex].answers.filter(
      answer => answer.answerLabel === userChoice
    )
    setAnswerIndex(resultIndex)
    setOpenModal(true)
    const quizUpdated = await quizService.updateUserQuiz(
      userChallenge.quiz,
      result[0].isTrue,
      questionIndex
    )
    const userChallengeUpdated = await challengeService.updateUserChallenge(
      userChallenge,
      UserChallengeUpdateFlag.QUIZ_UPDATE,
      quizUpdated
    )
    dispatch(updateUserChallengeList(userChallengeUpdated))
  }

  const handleChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setUserChoice(e.target.value)
  }
  const goNextQuestion = useCallback(() => {
    setUserChoice('')
    setOpenModal(false)
    if (questionIndex === userChallenge.quiz.questions.length - 1) {
      setIsCustomQuest(true)
    }
    if (questionIndex !== userChallenge.quiz.questions.length - 1) {
      setQuestionIndex(questionIndex + 1)
    }
  }, [
    questionIndex,
    setIsCustomQuest,
    setQuestionIndex,
    setUserChoice,
    setOpenModal,
    userChallenge.quiz.questions.length,
  ])

  return (
    <div className="quiz-container">
      <div className="question-container">
        <StyledIconButton
          icon={CloseIcon}
          onClick={goBack}
          aria-label={t('quiz.accessibility.window_quit')}
          className="btn-back"
        />
        <p className="index-question">{questionIndex + 1}/5</p>
        <p className="question text-18-bold">
          {userChallenge.quiz.questions[questionIndex].questionLabel}
        </p>
        {userChallenge.quiz.questions[questionIndex].answers.map(
          (answer, index) => (
            <div className="answer" key={index}>
              <input
                type="radio"
                id={`answer${index}`}
                value={answer.answerLabel}
                onChange={handleChange}
                checked={userChoice === answer.answerLabel}
              />
              <label htmlFor={`answer${index}`} className="text-16-bold">
                {answer.answerLabel}
              </label>
            </div>
          )
        )}
      </div>
      <Button
        aria-label={t('quiz.accessibility.button_validate')}
        onClick={validateQuestion}
        disabled={!userChoice}
        className="btnSecondary validate"
      >
        {t('quiz.button_validate')}
      </Button>
      {openModal && (
        <QuizExplanationModal
          open={openModal}
          answerIndex={answerIndex}
          question={userChallenge.quiz.questions[questionIndex]}
          goNext={goNextQuestion}
          handleCloseClick={() => setOpenModal(false)}
        />
      )}
    </div>
  )
}

export default QuizQuestionContent
