import { QueryResult } from 'cozy-client'
import { FluidType, UserDuelState } from 'enums'
import { DateTime, Duration } from 'luxon'
import { DuelEntity, UserDuel } from 'models'
import DuelService from 'services/duel.service'
import { fullGraphData, graphData } from 'tests/__mocks__/chartData.mock'
import mockClient from 'tests/__mocks__/client.mock'
import {
  allDuelData,
  duelData,
  duelEntity,
} from 'tests/__mocks__/duelData.mock'
import {
  fluidStatusConnectedData,
  fluidStatusData,
} from 'tests/__mocks__/fluidStatusData.mock'
import { getError } from 'tests/__mocks__/testUtils'

const mockGetPerformanceIndicators = jest.fn()
const mockGetGraphData = jest.fn()

jest.mock('./consumption.service', () => {
  return jest.fn(() => ({
    getPerformanceIndicators: mockGetPerformanceIndicators,
    getGraphData: mockGetGraphData,
  }))
})

const mockAggregatePerformanceIndicators = jest.fn()
jest.mock('./performanceIndicator.service', () => {
  return jest.fn(() => ({
    aggregatePerformanceIndicators: mockAggregatePerformanceIndicators,
  }))
})

describe('Duel service', () => {
  const duelService = new DuelService(mockClient)

  describe('getAllDuelEntities method', () => {
    it('should return all duel entity', async () => {
      const mockQueryResult: QueryResult<DuelEntity[]> = {
        data: allDuelData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await duelService.getAllDuelEntities()
      expect(result).toEqual(allDuelData)
    })
  })

  describe('getDuelEntityById method', () => {
    it('should return one duel by id', async () => {
      const mockQueryResult: QueryResult<DuelEntity[]> = {
        data: [duelData],
        bookmark: '',
        next: false,
        skip: 0,
      }
      const duelId = 'DUEL001'
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await duelService.getDuelEntityById(duelId)
      expect(result).toEqual(duelData)
    })
  })

  describe('deleteAllDuelEntities method', () => {
    it('should delete all duel entities', async () => {
      const mockQueryResult: QueryResult<DuelEntity[]> = {
        data: allDuelData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)

      const result = await duelService.deleteAllDuelEntities()
      expect(result).toBeTruthy()
    })

    it('should throw an error because destroy failed', async () => {
      const mockQueryResult: QueryResult<DuelEntity[]> = {
        data: allDuelData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      mockClient.destroy.mockRejectedValue(new Error())
      const error = await getError(async () =>
        duelService.deleteAllDuelEntities()
      )
      expect(error).toEqual(new Error())
    })
  })

  describe('startUserDuel method', () => {
    it('should return the userDuel with onGoing state', async () => {
      jest
        .spyOn(DateTime, 'local')
        .mockReturnValue(
          DateTime.fromISO('2020-10-01T00:00:00.000Z', { zone: 'utc' })
        )
      const result = await duelService.startUserDuel(duelData)

      const mockUpdatedDuel: UserDuel = {
        ...duelData,
        state: UserDuelState.ONGOING,
        startDate: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
          zone: 'utc',
        }),
      }
      expect(result).toEqual(mockUpdatedDuel)
    })
  })

  describe('resetUserDuel method', () => {
    it('should return the userDuel with unlocked state', async () => {
      const result = await duelService.resetUserDuel(duelData)
      const mockUpdatedDuel: UserDuel = {
        ...duelData,
        startDate: null,
        state: UserDuelState.UNLOCKED,
        threshold: 0,
        userConsumption: 0,
      }
      expect(result).toEqual(mockUpdatedDuel)
    })
  })

  describe('parseDuelEntityToDuel method', () => {
    it('should return the userDuel from a duelEntity', () => {
      const mockUpdatedDuel: UserDuel = {
        ...duelEntity,
        state: UserDuelState.LOCKED,
        startDate: null,
        threshold: 0,
        fluidTypes: [],
        userConsumption: 0,
      }

      const result = duelService.parseDuelEntityToDuel(duelEntity)
      expect(result).toEqual(mockUpdatedDuel)
    })
  })

  describe('updateUserDuelThreshold method', () => {
    it('should return the userDuel with an updated threshold and fluidtypes', async () => {
      mockGetGraphData.mockResolvedValue(fullGraphData)
      mockGetPerformanceIndicators.mockResolvedValue([])
      const aggregatePerformanceIndicators = {
        value: 55,
        compareValue: null,
        percentageVariation: null,
      }
      mockAggregatePerformanceIndicators.mockReturnValue(
        aggregatePerformanceIndicators
      )
      const mockUpdatedDuel: UserDuel = {
        ...duelData,
        state: UserDuelState.UNLOCKED,
        threshold: 55,
        fluidTypes: [FluidType.ELECTRICITY, FluidType.WATER, FluidType.GAS],
      }
      const result = await duelService.updateUserDuelThreshold(
        duelData,
        fluidStatusConnectedData
      )
      expect(result).toEqual(mockUpdatedDuel)
    })
    it('should return the userDuel with the state to NO_REF_PERIOD_VALID', async () => {
      mockGetGraphData.mockResolvedValueOnce(graphData)
      mockGetPerformanceIndicators.mockResolvedValueOnce([])
      const aggregatePerformanceIndicators = {
        value: 55,
        compareValue: null,
        percentageVariation: null,
      }
      mockAggregatePerformanceIndicators.mockReturnValue(
        aggregatePerformanceIndicators
      )
      const mockUpdatedDuel: UserDuel = {
        ...duelData,
        state: UserDuelState.NO_REF_PERIOD_VALID,
        duration: Duration.fromObject({ years: 99 }),
      }
      const falseDuelData: UserDuel = {
        ...duelData,
        duration: Duration.fromObject({ years: 99 }),
      }
      const result = await duelService.updateUserDuelThreshold(
        falseDuelData,
        fluidStatusData
      )
      expect(result).toEqual(mockUpdatedDuel)
    })
  })

  describe('getDuelFromDuelEntities method', () => {
    it('should return the userDuel from a duelEntity', () => {
      const searchId = 'DUEL001'

      const result = duelService.getDuelFromDuelEntities(allDuelData, searchId)
      expect(result).toEqual(duelData)
    })
  })
})
