import * as Sentry from '@sentry/react'
import { Client } from 'cozy-client'
import logger from 'cozy-logger'
import { REMOTE_ORG_ECOLYO_AGENT, REMOTE_ORG_ECOLYO_AGENT_REC } from 'doctypes'
import { FluidType, TimeStep } from 'enums'
import get from 'lodash/get'
import { DateTime, DurationLike } from 'luxon'
import mjml2html from 'mjml'
import { MonthlyReport, PerformanceIndicator } from 'models'
import ConsumptionService from 'services/consumption.service'
import EnvironmentService from 'services/environment.service'
import MailService from 'services/mail.service'
import { PermissionsService } from 'services/permissions.service'
import ProfileService from 'services/profile.service'
import { getMonthNameWithPrep } from 'utils/utils'
import { runService } from './service'
const monthlyReportTemplate = require('notifications/monthlyReport.hbs')

const logStack = logger.namespace('report')

interface MonthlyReportNotificationProps {
  client: Client
}

/**
 * Get consumption value for all fluid if exist
 * @param fluidTypes - FluidType
 */
const getConsumptionValue = async (
  client: Client,
  fluidTypes: FluidType[],
  period: 'month' | 'year'
): Promise<PerformanceIndicator[]> => {
  const consumptionService = new ConsumptionService(client)
  const analysisDate = DateTime.local().setZone('utc', { keepLocalTime: true })
  const timePeriod = {
    startDate: analysisDate.minus({ month: 1 }).startOf('month'),
    endDate: analysisDate.minus({ month: 1 }).endOf('month'),
  }
  const minusPeriod: DurationLike =
    period === 'year' ? { year: 1, month: 1 } : { month: 2 }

  const comparisonTimePeriod = {
    startDate: analysisDate.minus(minusPeriod).startOf('month'),
    endDate: analysisDate.minus(minusPeriod).endOf('month'),
  }

  return consumptionService.getPerformanceIndicators(
    timePeriod,
    TimeStep.MONTH,
    fluidTypes,
    comparisonTimePeriod
  )
}

/**
 * Build consumption text to attach in email. According to client consumption, display electricity and/or gaz and/or water
 */
const buildComparisonText = async (
  client: Client,
  period: 'month' | 'year'
) => {
  logStack('info', 'Building consumption text...')
  const consumption = await getConsumptionValue(
    client,
    [FluidType.ELECTRICITY, FluidType.GAS, FluidType.WATER],
    period
  )
  const fluidTexts = []

  if (consumption[FluidType.ELECTRICITY]?.percentageVariation) {
    const value = consumption[FluidType.ELECTRICITY].percentageVariation
    const sign = value > 0 ? '+' : '-'
    fluidTexts.push(
      `<p class="elec text">${sign}&nbsp;${formatConsumptionValue(
        value
      )}&nbsp;%&nbsp;d'électricité</p>`
    )
  }

  if (consumption[FluidType.GAS]?.percentageVariation) {
    const value = consumption[FluidType.GAS].percentageVariation
    const sign = value > 0 ? '+' : '-'
    fluidTexts.push(
      `<p class="gas text">${sign}&nbsp;${formatConsumptionValue(
        value
      )}&nbsp;%&nbsp;de&nbsp;gaz</p>`
    )
  }

  if (consumption[FluidType.WATER]?.percentageVariation) {
    const value = consumption[FluidType.WATER].percentageVariation
    const sign = value > 0 ? '+' : '-'
    fluidTexts.push(
      `<p class="water text">${sign}&nbsp;${formatConsumptionValue(
        value
      )}&nbsp;%&nbsp;d'eau</p>`
    )
  }

  return fluidTexts.join('')
}

/**
 * Get the ceiled absolute percentage representation of a number
 * @example (ex: -0.1234 => 13)
 */
const formatConsumptionValue = (value: number): number => {
  return Math.ceil(Math.abs(value * 100))
}

/**
 * getMonthlyReport
 */
const getMonthlyReport = async (
  year: string,
  month: string,
  client: Client
): Promise<MonthlyReport> => {
  try {
    const environmentService = new EnvironmentService()
    logStack(
      'info',
      environmentService.isProduction()
        ? 'Fetching data from BO prod'
        : 'Fetching data from BO rec'
    )

    const result = await client
      .getStackClient()
      .fetchJSON(
        'GET',
        environmentService.isProduction()
          ? `${REMOTE_ORG_ECOLYO_AGENT}?year=${year}&month=${month}`
          : `${REMOTE_ORG_ECOLYO_AGENT_REC}?year=${year}&month=${month}`
      )
    return result
  } catch (error) {
    logStack('error', JSON.stringify(error))
    Sentry.captureException(error)
    return {
      year: parseInt(year),
      month: parseInt(month),
      subject: '',
      info: '',
      image: '',
      newsTitle: 'Les nouveautés du service',
      newsContent: '',
      question: '',
      link: '',
    }
  }
}

const monthlyReportNotification = async ({
  client,
}: MonthlyReportNotificationProps) => {
  logStack('info', 'Fetching user profile...')
  const profileService = new ProfileService(client)
  const permissionsService = new PermissionsService(client)
  const userProfile = await profileService.getProfile()
  if (!userProfile?.sendAnalysisNotification) {
    logStack(
      'info',
      'End of process - Report Notification disabled in user profile'
    )
    return
  }

  // Init mail token for user in case he don't have one
  if (!userProfile.mailToken || userProfile.mailToken === '') {
    const token: string = require('crypto').randomBytes(48).toString('hex')

    try {
      await profileService.updateProfile({
        mailToken: token,
      })
    } catch (error) {
      logStack('error', 'Update mailToken user profile error : ' + error)
      Sentry.captureException(error)
      throw error
    }
  }

  let username = ''
  let analysisLink = ''

  logStack('info', 'Fetching data for mail...')
  // Retrieve public name from the stack
  const settings = await client
    .getStackClient()
    .fetchJSON('GET', '/settings/instance')
  const publicName = get(settings, 'data.attributes.public_name')
  if (publicName) {
    username = publicName
  }

  // Retrieve link to ecolyo app from the stack
  const apps = await client.getStackClient().fetchJSON('GET', '/apps/ecolyo')
  const appLink = get(apps, 'data.links.related')
  if (appLink) {
    analysisLink = appLink
  }

  logStack('info', 'Creation of mail...')
  const mailService = new MailService()

  const today = DateTime.local().setZone('utc', {
    keepLocalTime: true,
  })
  const month = today.toFormat('MM')
  const year = today.toFormat('yyyy')
  const monthlyReport = await getMonthlyReport(year, month, client)

  /** bounce token, will be used to calculate bounce clicks from monthly report */
  let bounceToken = undefined
  if (userProfile?.mailToken) {
    bounceToken = userProfile.mailToken
  }

  if (!analysisLink.includes('analysis')) {
    analysisLink = analysisLink + '/#/analysis'
  }
  if (bounceToken) {
    analysisLink += '?token=' + bounceToken
  }

  let unsubscribeUrl = appLink + 'public'
  const unsubscribeToken = await permissionsService.getShareCode()
  if (unsubscribeToken) {
    unsubscribeUrl += `?token=${unsubscribeToken}`
  }

  const monthComparisonText = await buildComparisonText(client, 'month')
  const yearComparisonText = await buildComparisonText(client, 'year')

  const isInfo = monthlyReport.info !== ''
  const isServiceNews =
    monthlyReport.newsTitle !== '' && monthlyReport.newsContent !== ''
  const isPoll = monthlyReport.question !== '' && monthlyReport.link !== ''

  const date = DateTime.local()
    .setZone('utc', { keepLocalTime: true })
    .minus({ month: 1 })

  const environmentService = new EnvironmentService()
  const baseUrl = environmentService.getPublicURL()

  const template = monthlyReportTemplate({
    title: 'Infos & bilan consos',
    baseUrl: baseUrl,
    username: username,
    clientUrl: analysisLink,
    unsubscribeUrl: unsubscribeUrl,
    comparisonExist:
      monthComparisonText.length > 0 || yearComparisonText.length > 0,
    monthComparisonExist: monthComparisonText.length > 0,
    monthComparisonText: monthComparisonText,
    yearComparisonExist: yearComparisonText.length > 0,
    yearComparisonText: yearComparisonText,
    infoText: monthlyReport.info.replace(/{cozyUrl}/g, appLink + '#/'),
    infoImage:
      monthlyReport.image !== ''
        ? baseUrl + monthlyReport.image
        : baseUrl + '/assets/astuce.png',
    isContent: isInfo || isServiceNews || isPoll,
    isInfo: isInfo,
    divider1: (isInfo && isServiceNews) || (isInfo && isPoll),
    isServiceNews: isServiceNews,
    divider2: isServiceNews && isPoll,
    isPoll: isPoll,
    newsTitle: monthlyReport.newsTitle,
    newsContent: monthlyReport.newsContent.replace(
      /{cozyUrl}/g,
      appLink + '#/'
    ),
    pollText: monthlyReport.question.replace(/{cozyUrl}/g, appLink + '#/'),
    pollUrl: monthlyReport.link,
    previousMonth: getMonthNameWithPrep(date.minus({ month: 1 })),
    currentMonth: getMonthNameWithPrep(date),
    previousYear: date.year - 1,
    currentYear: date.year,
    previousMonthYear: date.month === 1 ? date.year - 1 : date.year,
    consoImageUrl: baseUrl + '/assets/multifluidConsumption.png',
    feedbackImageUrl: baseUrl + '/assets/feedback.png',
  })

  const mailData = {
    mode: 'campaign',
    subject: monthlyReport.subject,
    parts: [
      {
        type: 'text/html',
        body: mjml2html(template).html,
      },
    ],
  }
  logStack('info', 'Sending mail...')
  mailService.SendMail(client, mailData)
}

runService(monthlyReportNotification)
